@@ -7,7 +7,7 @@
 
 # python packages
 from absl.flags import FLAGS
-from typing import Optional
+from typing import Optional, Any
 
 # 3rd-party packages
 from loguru import logger
@@ -16,7 +16,7 @@ from loguru import logger
 # self-defined packages
 
 @logger.catch(reraise=True)
-def get_flag(attr: str, default: Optional[str] = None) -> str:
+def get_flag(attr: str, default: Optional[Any] = None) -> Any:
     try:
         if hasattr(FLAGS, attr):
             return getattr(FLAGS, attr)
