@@ -14,15 +14,25 @@ import pynvml
 
 
 class GPU:
-    def __init__(self):
+    @property
+    def min_available_fraction(self):
+        return self._min_available_fraction
+
+    @min_available_fraction.setter
+    def min_available_fraction(self, value):
+        self._min_available_fraction = value
+        self.visible_devices = [self.physical_devices[i] for i in range(len(self.physical_devices)) if
+                                self.check_available(i)]
+        logger.debug(f"available gpus: {self.visible_devices}")
+
+    def __init__(self, min_available_fraction=0.7):
         import tensorflow as tf
         self.config = tf.config
         self.physical_devices = tf.config.list_physical_devices('GPU')
         logger.debug(f"physical_devices gpus: {self.physical_devices}")
-        self.visible_devices = [self.physical_devices[i] for i in range(len(self.physical_devices)) if
-                                self.check_available(i)]
-        logger.debug(f"available gpus: {self.visible_devices}")
+        self._min_available_fraction = min_available_fraction
         self.auto_select_free = True
+        self.min_available_fraction = min_available_fraction
 
     def get_memory_info(self, idx: int) -> Any:
         pynvml.nvmlInit()
@@ -31,7 +41,9 @@ class GPU:
         pynvml.nvmlShutdown()
         return meminfo
 
-    def check_available(self, idx: int, fraction=0.7) -> bool:
+    def check_available(self, idx: int, fraction=None) -> bool:
+        if fraction is None:
+            fraction = self.min_available_fraction
         meminfo = self.get_memory_info(idx)
         used = meminfo.used / meminfo.total
         if used > fraction:
@@ -39,23 +51,23 @@ class GPU:
         else:
             return True
 
-    def set_visible_device(self, visible_device_indexes: Union[int, List[int]]) -> None:
-        if isinstance(visible_device_indexes, int):
-            visible_device_indexes = [visible_device_indexes]
-        visible_devices = []
-        for i in visible_device_indexes:
-            if self.physical_devices[i] in self.visible_devices:
-                visible_devices.append(self.physical_devices[i])
-        if self.auto_select_free and len(visible_devices) < len(visible_device_indexes):
-            logger.debug("Not enough free gpu in selection, automatically select free gpu(s).")
-            free_gpu_idxes = [j for j in range(len(self.physical_devices)) if
-                              j not in visible_device_indexes and self.check_available(j)]
-            visible_devices += [self.physical_devices[free_gpu_idx] for free_gpu_idx in
-                                free_gpu_idxes[:(len(visible_device_indexes) - len(visible_devices))]]
+    def set_visible_device(self, target_device_indexes: Optional[Union[int, List[int]]]) -> None:
+        if isinstance(target_device_indexes, int):
+            target_device_indexes = [target_device_indexes]
 
-        self.config.experimental.set_visible_devices(devices=visible_devices, device_type='GPU')
-        self.visible_devices = visible_devices
-        logger.debug(f"Set available gpus: {visible_devices}")
+        selected_visible_devices = []
+        if target_device_indexes:
+            for i in target_device_indexes:
+                if target_device_indexes[i] < len(self.visible_devices):
+                    selected_visible_devices.append(self.visible_devices[i])
+            if self.auto_select_free and len(selected_visible_devices) < len(target_device_indexes):
+                logger.debug("Not enough free gpu in selection, automatically select free gpu(s).")
+                selected_visible_devices += self.visible_devices[
+                                            :(len(target_device_indexes) - len(selected_visible_devices))]
+
+        self.config.experimental.set_visible_devices(devices=selected_visible_devices, device_type='GPU')
+        self.visible_devices = selected_visible_devices
+        logger.debug(f"Set available gpus: {selected_visible_devices}")
 
     def set_allow_growth(self, allow_growth: bool) -> None:
         logger.debug(f"Set gpu memory_growth to be : {allow_growth}")
