@@ -18,14 +18,45 @@ class LogHistory(keras.callbacks.Callback):
         pass
 
     def on_epoch_end(self, epoch, logs=None):
-        s = f"[Epoch {epoch+1}/{self.params['epochs']}]"
+        s = f"[Epoch {epoch + 1}/{self.params['epochs']}]"
         for key, value in logs.items():
             s += f" {key}:{value:.4f}"
 
         logger.debug(s)
 
 
-def plot_history(history, save_path=None):
+class PlotHistory(keras.callbacks.Callback):
+    def __init__(self, loss=None, val_loss=None):
+        super().__init__()
+        if not loss:
+            loss = []
+        if not val_loss:
+            val_loss = []
+
+        self.loss = loss
+        self.val_loss = val_loss
+
+    def on_batch_begin(self, batch, logs=None):
+        pass
+
+    def on_epoch_end(self, epoch, logs=None):
+        self.loss.append(logs.get("loss", 0))
+        self.val_loss.append(logs.get("val_loss", 0))
+
+    def plot(self, save_path=None):
+        plt.plot(self.loss)
+        plt.plot(self.val_loss)
+        plt.title('model loss')
+        plt.ylabel('loss')
+        plt.xlabel('epoch')
+        plt.legend(['train', 'val'], loc='upper left')
+        if save_path:
+            plt.savefig(save_path)
+
+        plt.show()
+
+
+def plot_history(history, save_path=None) -> None:
     plt.plot(history.history['loss'])
     plt.plot(history.history['val_loss'])
     plt.title('model loss')
@@ -38,9 +69,7 @@ def plot_history(history, save_path=None):
     plt.show()
 
 
-
-
-def get_balanced_weight(dic_class):
+def get_balanced_weight(dic_class: dict) -> dict:
     total = 0
     dic_weight = {}
 
