@@ -6,8 +6,15 @@
 """
 
 # python packages
+# Python 3.7 introduces PEP 563: postponed evaluation of annotations.
+# A module that uses the future statement from __future__ import annotations
+# will store annotations as strings automatically:
+from __future__ import annotations
+
 import abc
 import json
+from typing import Optional, List, Union, Tuple, Dict
+from collections import defaultdict
 
 # 3rd-party packages
 import cv2
@@ -15,7 +22,8 @@ import imgaug as ia
 from loguru import logger
 
 # self-defined packages
-from utils import os_path, image_processing
+from utils import os_path
+from utils.my_class import type_myclass
 
 ANNOTATION_FOLDER_DEFAULT_NAME = "Annotations"
 
@@ -43,19 +51,19 @@ class VOCLabel(metaclass=abc.ABCMeta):
 
 class VOCBbox(VOCLabel):
     @property
-    def width(self):
+    def width(self) -> float:
         return self._width
 
     @property
-    def height(self):
+    def height(self) -> float:
         return self._height
 
     @property
-    def xmin(self):
+    def xmin(self) -> float:
         return self._xmin
 
     @xmin.setter
-    def xmin(self, value):
+    def xmin(self, value: float) -> None:
         if hasattr(self, "xmax"):
             assert value < self.xmax, f"Cannot set property xmax, {value} > {self.xmax}"
         self._xmin = value
@@ -64,40 +72,40 @@ class VOCBbox(VOCLabel):
             assert self.width > 0, f"width: {self._width} < 0"
 
     @property
-    def x1(self):
+    def x1(self) -> float:
         return self._xmin
 
     @property
-    def x2(self):
+    def x2(self) -> float:
         return self._xmax
 
     @property
-    def y1(self):
+    def y1(self) -> float:
         return self._ymin
 
     @property
-    def y2(self):
+    def y2(self) -> float:
         return self._ymax
 
     @property
-    def xmax(self):
+    def xmax(self) -> float:
         return self._xmax
 
     @xmax.setter
-    def xmax(self, value):
+    def xmax(self, value: float) -> None:
         if hasattr(self, "xmin"):
             assert value > self.xmin, f"Cannot set property xmin, {value} < {self.xmin}"
         self._xmax = value
         if hasattr(self, "xmin"):
             self._width = self.xmax - self.xmin
-            assert self.width > 0 ,f"width: {self._width} < 0"
+            assert self.width > 0, f"width: {self._width} < 0"
 
     @property
-    def ymin(self):
+    def ymin(self) -> float:
         return self._ymin
 
     @ymin.setter
-    def ymin(self, value):
+    def ymin(self, value: float) -> None:
         if hasattr(self, "ymax"):
             assert value < self.ymax, f"Cannot set property ymin, {value} > {self.ymax}"
         self._ymin = value
@@ -105,20 +113,22 @@ class VOCBbox(VOCLabel):
             self._height = self.ymax - self.ymin
 
     @property
-    def ymax(self):
+    def ymax(self) -> float:
         return self._ymax
 
     @ymax.setter
-    def ymax(self, value):
+    def ymax(self, value: float) -> None:
         self._ymax = value
         if hasattr(self, "ymin"):
             assert value > self.ymin, f"Cannot set property ymax, {value} < {self.ymin}"
             self._height = self.ymax - self.ymin
 
-    def get_coords(self, format=None):
-        if format is None:
+    def get_coords(self, format_: str = None) -> Union[Tuple[float, float, float, float],
+                                                       Dict[str, float],
+                                                       Tuple[Tuple[float, float], Tuple[float, float]]]:
+        if format_ is None:
             return self.x1, self.y1, self.x2, self.y2
-        elif format == "dict":
+        elif format_ == "dict":
             return {"x1": self.x1, "y1": self.y1, "x2": self.x2, "y2": self.y2}
         else:
             return (self.x1, self.y1), (self.x2, self.y2)
@@ -138,23 +148,23 @@ class VOCBbox(VOCLabel):
         self.ymin = int(ymin)
         self.ymax = int(ymax)
 
-    def to_dic(self, recursively=True):
+    def to_dic(self, recursively: bool = True) -> Union[VOCBbox, Dict[str, float]]:
         if not recursively:
             return self
         return {"xmin": self.xmin, "ymin": self.ymin,
                 "xmax": self.xmax, "ymax": self.ymax}
 
-    def to_json_file(self, filepath):
+    def to_json_file(self, filepath: str) -> None:
         with open(filepath, "w+") as f:
             f.write(json.dumps(self.to_dic()))
         logger.debug(f"Json {filepath} exported.")
 
-    def to_imgaug_bbox(self, label=None):
+    def to_imgaug_bbox(self, label: Optional[str] = None) -> ia.BoundingBox:
         imgaug_bbox = ia.BoundingBox(x1=self.xmin, y1=self.ymin, x2=self.xmax, y2=self.ymax, label=label)
         return imgaug_bbox
 
     @staticmethod
-    def from_imgaug_bbox(imgaug_bbox):
+    def from_imgaug_bbox(imgaug_bbox: ia.BoundingBox) -> VOCBbox:
         assert isinstance(imgaug_bbox, ia.BoundingBox)
         voc_bbox = VOCBbox(imgaug_bbox.x1, imgaug_bbox.y1, imgaug_bbox.x2, imgaug_bbox.y2)
         return voc_bbox
@@ -162,29 +172,29 @@ class VOCBbox(VOCLabel):
 
 class VOCObject(VOCLabel):
     @property
-    def truncated(self):
+    def truncated(self) -> bool:
         return self._truncated
 
     @truncated.setter
-    def truncated(self, value):
+    def truncated(self, value: bool) -> None:
         assert isinstance(value, bool)
         self._truncated = value
 
     @property
-    def difficult(self):
-        return self._truncated
+    def difficult(self) -> bool:
+        return self._difficult
 
     @difficult.setter
-    def difficult(self, value):
+    def difficult(self, value: bool) -> None:
         assert isinstance(value, bool)
         self._difficult = value
 
     @property
-    def defined_classes(self):
+    def defined_classes(self) -> Union[type_myclass, List[type_myclass]]:
         return self._defined_classes
 
     @defined_classes.setter
-    def defined_classes(self, value):
+    def defined_classes(self, value: Union[type_myclass, List[type_myclass]]) -> None:
         self._defined_classes = value
         if value is None:
             return
@@ -204,59 +214,62 @@ class VOCObject(VOCLabel):
                     pass
 
     @property
-    def name(self):
+    def name(self) -> Union[str, List[str], type_myclass, List[type_myclass]]:
         return self._name
 
     @name.setter
-    def name(self, value):
+    def name(self, value: Union[str, List[str], type_myclass, List[type_myclass]]):
         self._name = value
 
     @property
-    def bbox(self):
+    def bbox(self) -> VOCBbox:
         return self._bbox
 
     @bbox.setter
-    def bbox(self, value):
+    def bbox(self, value: Union[VOCBbox, list]) -> None:
         if isinstance(value, VOCBbox):
             self._bbox = value
         else:
             self._bbox = VOCBbox(*value)
 
     @property
-    def x1(self):
+    def x1(self) -> float:
         return self._bbox.x1
 
     @property
-    def x2(self):
+    def x2(self) -> float:
         return self._bbox.x2
 
     @property
-    def y1(self):
+    def y1(self) -> float:
         return self._bbox.y1
 
     @property
-    def y2(self):
+    def y2(self) -> float:
         return self._bbox.y2
 
     @property
-    def score(self):
+    def score(self) -> Union[List[float], float]:
         return self._score
 
     @score.setter
-    def score(self, value):
+    def score(self, value: Union[List[float], float]):
         self._score = value
 
     @property
-    def from_annotation(self):  # parent
+    def from_annotation(self) -> VOCAnnotation:  # parent
         return self._voc_annotation
 
     @from_annotation.setter
-    def from_annotation(self, value):
+    def from_annotation(self, value: VOCAnnotation):
         assert isinstance(value, VOCAnnotation) or value is None
         self._voc_annotation = value
 
-    def __init__(self, name, bbox, truncated=False, difficult=False, defined_classes=None, from_annotation=None,
-                 score=1.0):
+    def __init__(self, name: Union[str, List[str], type_myclass, List[type_myclass]],
+                 bbox: Union[VOCBbox, list],
+                 truncated: bool = False, difficult: bool = False,
+                 defined_classes: bool = None, from_annotation: bool = None,
+                 score: float = 1.0):
         self.name = name
         self.bbox = bbox
         self.defined_classes = defined_classes
@@ -265,7 +278,7 @@ class VOCObject(VOCLabel):
         self.from_annotation = from_annotation
         self.score = score
 
-    def to_dic(self, recursively=True):
+    def to_dic(self, recursively: bool = True) -> dict:
         if isinstance(self.name, list):
             name = [str(k) for k in self.name]
         else:
@@ -275,13 +288,14 @@ class VOCObject(VOCLabel):
                 "truncated": self.truncated,
                 "difficult": self.difficult}
 
-    def to_json_file(self, filepath):
+    def to_json_file(self, filepath: str) -> None:
         with open(filepath, "w+") as f:
             f.write(json.dumps(self.to_dic(recursively=True), cls=VOCJSONEncoder))
         logger.debug(f"Json {filepath} exported.")
 
     @staticmethod
-    def load_json(j, defined_classes=None, from_annotation=None, debug=True):
+    def load_json(j, defined_classes: Union[type_myclass, List[type_myclass]] = None,
+                  from_annotation: Optional[VOCAnnotation] = None, debug: bool = True) -> VOCObject:
         bbox = VOCBbox(**j["bndbox"])
         obj = VOCObject(name=j["name"], bbox=bbox, truncated=j["truncated"], difficult=j["difficult"],
                         defined_classes=defined_classes, from_annotation=from_annotation)
@@ -289,13 +303,13 @@ class VOCObject(VOCLabel):
             logger.debug(f"VOCObject loaded from {j}")
         return obj
 
-    def to_imgaug_bbox(self):
+    def to_imgaug_bbox(self) -> ia.BoundingBox:
         imgaug_bbox = ia.BoundingBox(x1=self.bbox.xmin, y1=self.bbox.ymin, x2=self.bbox.xmax, y2=self.bbox.ymax,
                                      label=self.name)
         return imgaug_bbox
 
     @staticmethod
-    def from_imgaug_bbox(imgaug_bbox):
+    def from_imgaug_bbox(imgaug_bbox) -> VOCObject:
         assert isinstance(imgaug_bbox, ia.BoundingBox)
         bbox = VOCBbox(imgaug_bbox.x1, imgaug_bbox.y1, imgaug_bbox.x2, imgaug_bbox.y2)
         voc_obj = VOCObject(imgaug_bbox.label, bbox)
@@ -304,11 +318,11 @@ class VOCObject(VOCLabel):
 
 class VOCAnnotation(VOCLabel):
     @property
-    def image_path(self):  # image_path
+    def image_path(self) -> str:  # image_path
         return self._image_path
 
     @image_path.setter
-    def image_path(self, value):
+    def image_path(self, value: str) -> None:
         self._org_image_path = value
         self._image_path = value
         if self._image_path is not None:
@@ -316,21 +330,21 @@ class VOCAnnotation(VOCLabel):
             self._org_image_path = os_path.abspath(self._org_image_path)
 
     @property
-    def annotation_path(self):
+    def annotation_path(self) -> Union[None, str]:
         if self._annotation_path:
             return self._annotation_path
-        elif self.filename:  # default value
+        elif self.filename and self.folder:  # default value
             return os_path.join(os_path.dirname(self.folder, full_path=True), ANNOTATION_FOLDER_DEFAULT_NAME,
                                 self.filename + ".json")
         return None
 
     @annotation_path.setter
-    def annotation_path(self, value):
+    def annotation_path(self, value: str) -> None:
         self._org_annotation_path = value
         self._annotation_path = value
 
     @property
-    def filename(self):
+    def filename(self) -> Union[None, str]:
         if self._annotation_path:
             return os_path.get_filename(self._annotation_path)
         if self.image_path:
@@ -338,66 +352,71 @@ class VOCAnnotation(VOCLabel):
         return None
 
     @property
-    def folder(self):
+    def folder(self) -> Union[None, str]:
+        if self._folder:
+            return self._folder
         if self.image_path:
             return os_path.dirname(self.image_path)
-            # return os_path.dirname(self.image_path, depth=2, full_path=False)
         if self.annotation_path:
             return os_path.dirname(self.annotation_path, depth=2)
-            # return os_path.dirname(self.annotation_path, depth=2, full_path=False)
         return None
 
+    @folder.setter
+    def folder(self, value) -> None:
+        self._folder = value
+
     @property
-    def objects(self):
+    def objects(self) -> List[VOCObject]:
         return self._objects
 
     @property
-    def bboxes(self):
+    def bboxes(self) -> List[VOCBbox]:
         bboxes = [obj.bbox for obj in self.objects]
         return bboxes
 
     @property
-    def size(self):
+    def size(self) -> tuple:
         return self._size
 
     @size.setter
-    def size(self, value):
-        value_int = tuple([int(v) for v in value])
-        self._size = value_int
+    def size(self, value: Union[list, tuple]):
+        if value:
+            value_int = tuple([int(v) for v in value])
+            self._size = value_int
 
     @property
-    def segmented(self):
+    def segmented(self) -> bool:
         return self._segmented
 
     @segmented.setter
-    def segmented(self, value):
+    def segmented(self, value: bool) -> None:
         self._segmented = value
 
     @property
-    def defined_classes(self):
+    def defined_classes(self) -> Union[type_myclass, List[type_myclass]]:
         return self._defined_classes
 
     @property
-    def from_annotation_set(self):  # parent
+    def from_annotation_set(self) -> VOCAnnotationSet:  # parent
         return self._from_annotation_set
 
     @from_annotation_set.setter
-    def from_annotation_set(self, value):
+    def from_annotation_set(self, value: VOCAnnotationSet) -> None:
         self._from_annotation_set = value
 
     @property
-    def for_classification(self):  # parent
+    def for_classification(self) -> bool:  # parent
         return self._for_classification
 
     @for_classification.setter
-    def for_classification(self, value):
+    def for_classification(self, value: bool) -> None:
         self._for_classification = value
 
     @property
-    def unpacked(self):
+    def unpacked(self) -> bool:
         return self._unpacked
 
-    def unpack(self):
+    def unpack(self) -> None:
         image = cv2.imread(self.image_path)
         self.size = image.shape
         self._unpacked = True
@@ -406,6 +425,7 @@ class VOCAnnotation(VOCLabel):
                  defined_classes=None, folder=None, filename=None, for_classification=False, size=None):
         # assert (image_path or (folder and filename))
 
+        self._folder = None
         if not image_path and (folder and filename):
             image_path = os_path.join(folder, filename)
 
@@ -424,17 +444,17 @@ class VOCAnnotation(VOCLabel):
         self.annotation_path = annotation_path
         self.for_classification = for_classification
 
-    def __len__(self):
+    def __len__(self) -> int:
         return len(self.objects)
 
-    def add_objects(self, objs):
+    def add_objects(self, objs) -> None:
         if not isinstance(objs, list):
             objs = [objs]
         for obj in objs:
             assert isinstance(obj, VOCObject)
             self._objects.append(obj)
 
-    def to_dic(self, recursively=True):
+    def to_dic(self, recursively: bool = True) -> dict:
         if not self.size:
             size = (0, 0, 0)
         else:
@@ -453,7 +473,7 @@ class VOCAnnotation(VOCLabel):
                 "segmented": self.segmented
                 }
 
-    def to_json_file(self, dir_=None, filepath=None):
+    def to_json_file(self, dir_: Optional[str] = None, filepath: Optional[str] = None) -> None:
         if dir_ is not None and filepath is None:
             filepath = os_path.join(dir_, self.filename)
         elif dir_ is None and filepath is None:
@@ -466,7 +486,11 @@ class VOCAnnotation(VOCLabel):
         logger.debug(f"Json {filepath} exported.")
 
     @staticmethod
-    def load_json(j, annotation_path=None, defined_classes=None, from_annotation_set=None, debug=True, image_dir=None):
+    def load_json(j, annotation_path: str = None,
+                  defined_classes: Optional[Union[type_myclass, List[type_myclass]]] = None,
+                  from_annotation_set: Optional[VOCAnnotationSet] = None,
+                  debug: bool = True,
+                  image_dir: Optional[str] = None) -> VOCAnnotation:
         if image_dir is not None:
             image_path = os_path.join(image_dir, j["filename"])
         else:
@@ -475,10 +499,11 @@ class VOCAnnotation(VOCLabel):
         # check image path exists
         if not os_path.exists(image_path):
             if annotation_path:
-                logger.warning(f"Image path {image_path} does not exist. Loading {annotation_path} failed.")
+                exception = f"Image path {image_path} does not exist. Loading {annotation_path} failed."
             else:
-                logger.warning(f"Image path {image_path} does not exist. Loading {j} failed.")
-            return None
+                exception = f"Image path {image_path} does not exist. Loading {j} failed."
+            logger.warning(exception)
+            raise FileExistsError(exception)
 
         ann = VOCAnnotation(image_path=image_path, from_annotation_set=from_annotation_set,
                             annotation_path=annotation_path, defined_classes=defined_classes,
@@ -501,13 +526,13 @@ class VOCAnnotation(VOCLabel):
         ann = VOCAnnotation.load_json(j, **kwargs)
         queue.put(ann)
 
-    def to_imgaug_bbox_in_image(self):
+    def to_imgaug_bbox_in_image(self) -> ia.BoundingBoxesOnImage:
         bbs = ia.BoundingBoxesOnImage([
             obj.to_imgaug_bbox() for obj in self.objects
         ], shape=self.size)
         return bbs
 
-    def flatten(self, for_classification=True):
+    def flatten(self, for_classification: bool = True) -> List[VOCAnnotation]:
         annotations = []
         for obj in self.objects:
             ann = VOCAnnotation(image_path=self.image_path, annotation_path=self.annotation_path,
@@ -518,9 +543,9 @@ class VOCAnnotation(VOCLabel):
 
     def read_image(self):
         if os_path.exists(self.image_path):
-            image = image_processing.imread(self.image_path)
+            image = cv2.imread(self.image_path)
         else:
-            image = image_processing.imread(self._org_image_path)
+            image = cv2.imread(self._org_image_path)
         if self.for_classification:
             bbox = self.objects[0].bbox
             x1, y1, x2, y2 = bbox.x1, bbox.y1, bbox.x2, bbox.y2
@@ -530,15 +555,15 @@ class VOCAnnotation(VOCLabel):
                 image = image[y1:y2 + 1, x1:x2 + 1]
         return image
 
-    def load_from_json_file(self, json_file):
-        with open(json_file) as f:
+    def load_from_json_file(self, json_file: str) -> None:
+        with open(json_file, "r") as f:
             j = json.load(f)
         self.__dict__.update(self.load_json(j, annotation_path=json_file).__dict__)
 
 
 class VOCAnnotationSet(VOCLabel):
     @property
-    def folder(self):
+    def folder(self) -> str:
         self._folder = self.annotations[0].folder
         for annotation in self.annotations:
             if annotation.folder != self._folder:
@@ -546,91 +571,99 @@ class VOCAnnotationSet(VOCLabel):
         return self._folder
 
     @folder.setter
-    def folder(self, value):
+    def folder(self, value: str) -> None:
         logger.warning(f"Changing all annotations folder to {value}")
         for i, annotation in enumerate(self.annotations):
             self.annotations[i].folder = value
 
     @property
-    def image_paths(self):  # image_paths
+    def image_paths(self) -> List[str]:  # image_paths
         return self._image_paths
 
     @property
-    def annotations(self):
+    def annotations(self) -> List[VOCAnnotation]:
         return self._annotations
 
     @property
-    def defined_classes(self):
+    def defined_classes(self) -> Union[type_myclass, List[type_myclass]]:
         return self._defined_classes
 
+    @defined_classes.setter
+    def defined_classes(self, value) -> None:
+        self._defined_classes = value
+        if value is not None:
+            for defined_class in self._defined_classes:
+                for klass in list(defined_class):
+                    self._class_count[defined_class][klass] = 0
+                    self._class_dict[defined_class][klass] = []
+
     @property
-    def class_count(self):
+    def class_count(self) -> dict:
+        if not self.defined_classes:
+            return self._class_count[str]
         return self._class_count
 
     @property
-    def class_dict(self):
+    def class_dict(self) -> dict:
+        if not self.defined_classes:
+            return self._class_dict[str]
         return self._class_dict
 
     def __len__(self):
         return len(self.annotations)
 
     @property
-    def unpacked(self):
+    def unpacked(self) -> bool:
         return self._unpacked
 
     @property
-    def for_classification(self):
+    def for_classification(self) -> bool:
         return self._for_classification
 
     @for_classification.setter
-    def for_classification(self, value):
+    def for_classification(self, value: bool) -> None:
         self._for_classification = value
 
-    def __init__(self, dir_or_file=None, defined_classes=None, debug=False,
-                 for_classification=False, image_dir=None):
+    def __init__(self, dir_or_file: Optional[Union[str, List[str]]] = None,
+                 defined_classes: Optional[Union[type_myclass, List[type_myclass]]] = None,
+                 debug: bool = False,
+                 for_classification: bool = False, image_dir: Optional[str] = None):
 
         self._folder = ""
         self._image_paths = []
         self._annotations = []
-        self._defined_classes = defined_classes
-        self._class_count = {}
-        self._class_dict = {}  # index of annotations
+        self._class_count = defaultdict(lambda: defaultdict(int))
+        self._class_dict = defaultdict(lambda: defaultdict(list))  # index of annotations
         self._unpacked = True
         self.for_classification = for_classification
         self.debug = debug
-
-        # init class_count, class_dict
-        if defined_classes:
-            for defined_class in defined_classes:
-                self._class_count[defined_class] = {}
-                self._class_dict[defined_class] = {}
-                for klass in list(defined_class):
-                    self._class_count[defined_class][klass] = 0
-                    self._class_dict[defined_class][klass] = []
+        self.defined_classes = defined_classes
 
         # load from dir or jsons or json
         if dir_or_file:
+            if not os_path.exists(dir_or_file):
+                raise FileNotFoundError
             if isinstance(dir_or_file, list) and all(os_path.isfile(x) for x in dir_or_file):
                 self.load_from_jsons(sorted(dir_or_file), debug=debug,
                                      for_classification=for_classification, image_dir=image_dir)
             elif os_path.isdir(dir_or_file) or \
                     (isinstance(dir_or_file, list) and all(os_path.isdir(x) for x in dir_or_file)):
                 if os_path.isdir(dir_or_file):
-                    label_files = os_path.list_dir(dir_or_file, extension=".json")
+                    label_files = os_path.list_dir(dir_or_file, extension=".json", exclude_pattern="._")
                 else:
                     label_files = []
                     for lf in dir_or_file:
-                        label_files += os_path.list_dir(lf, extension=".json")
+                        label_files += os_path.list_dir(lf, extension=".json", exclude_pattern="._")
                 if not label_files:
                     label_files = os_path.list_dir(os_path.join(dir_or_file, ANNOTATION_FOLDER_DEFAULT_NAME),
-                                                   extension=".json")
+                                                   extension=".json", exclude_pattern="._")
                 self.load_from_jsons(sorted(label_files), debug=debug, for_classification=for_classification,
                                      image_dir=image_dir)
             else:  # is file
-                raise NotImplemented
+                raise NotImplementedError
                 # self.load_from_json(dir_or_file)
 
-    def add_annotation(self, annotation):
+    def add_annotation(self, annotation: Union[List[VOCAnnotation], VOCAnnotation]) -> None:
         if not annotation:
             return
         if isinstance(annotation, list):
@@ -648,19 +681,12 @@ class VOCAnnotationSet(VOCLabel):
             for obj in annotation.objects:
                 if isinstance(obj.name, list):
                     for n in obj.name:
-                        if n in self._class_count:
-                            self._class_count[n] += 1
-                            self._class_dict[n].append(len(self))
-                        else:
-                            self._class_count[n] = 1
-                            self._class_dict[n] = [len(self)]
+                        self._class_count[str][n] += 1
+                        self._class_dict[str][n].append(len(self))
+
                 else:
-                    if obj.name in self._class_count:
-                        self._class_count[obj.name] += 1
-                        self._class_dict[obj.name].append(len(self))
-                    else:
-                        self._class_count[obj.name] = 1
-                        self._class_dict[obj.name] = [len(self)]
+                    self._class_count[str][obj.name] += 1
+                    self._class_dict[str][obj.name].append(len(self))
         else:
             for obj in annotation.objects:
                 if isinstance(obj.name, list):
@@ -671,10 +697,11 @@ class VOCAnnotationSet(VOCLabel):
                     self._class_count[type(obj.name)][obj.name] += 1
                     self._class_dict[type(obj.name)][obj.name].append(len(self))
 
-    def to_dic(self, recursively=True):
+    def to_dic(self, recursively: bool = True) -> List[dict]:
         return [annotation.to_dic(recursively) for annotation in self.annotations]
 
-    def to_json_files(self, dir_=None, filenames=None, filepaths=None):
+    def to_json_files(self, dir_: Optional[str] = None, filenames: Optional[List[str]] = None,
+                      filepaths: Optional[List[str]] = None) -> None:
         if filenames:
             assert len(filenames) == len(self._annotations)
         assert bool(dir_) is not bool(filepaths)
@@ -688,12 +715,13 @@ class VOCAnnotationSet(VOCLabel):
             else:
                 annotation.to_json_file()
 
-    def to_json_file(self, filename):
+    def to_json_file(self, filename: str) -> None:
         with open(filename, "w+") as f:
             f.write(json.dumps(self.to_dic(recursively=True), cls=VOCJSONEncoder))
         logger.debug(f"Json {filename} exported.")
 
-    def load_from_jsons(self, label_files, debug=False, for_classification=False, image_dir=None):
+    def load_from_jsons(self, label_files: List[str], debug: bool = False, for_classification: bool = False,
+                        image_dir: Optional[str] = None) -> VOCAnnotationSet:
 
         for label_file in label_files:
             with open(label_file, "r") as f:
@@ -709,7 +737,7 @@ class VOCAnnotationSet(VOCLabel):
         logger.debug(f"Loaded {len(self)} annotations from jsons {label_files[-5:-1]}")
         return self
 
-    def load_from_json(self, json_file):
+    def load_from_json(self, json_file: str) -> VOCAnnotationSet:
         with open(json_file, "r") as f:
             annotation_js = json.load(f)
 
@@ -719,9 +747,7 @@ class VOCAnnotationSet(VOCLabel):
         logger.debug(f"Loaded {len(self)} annotations from {json_file}.")
         return self
 
-    def unpack(self):
+    def unpack(self) -> None:
         for ann in self.annotations:
             ann.unpack()
         self._unpacked = True
-
-
