@@ -6,8 +6,7 @@
 """
 
 # python packages
-import timeit
-
+import time
 # 3rd-party packages
 from loguru import logger
 
@@ -17,16 +16,19 @@ from loguru import logger
 class Timer:
     def __init__(self, name=None):
         self.name = name
-        self.start_time = timeit.default_timer()
+        self.start_time = time.perf_counter()
         self.end_time = None
 
     def start(self):
-        self.start_time = timeit.default_timer()
+        self.start_time = time.perf_counter()
 
-    def stop(self):
-        self.end_time = timeit.default_timer()
+    def stop(self) -> float:
+        self.end_time = time.perf_counter()
+        time_elapsed = self.end_time - self.start_time
         logger.debug(
-            f"{self.name + ' ' if self.name is not None else ''} Time elapse {self.end_time - self.start_time}s")
+            f"{self.name + ' ' if self.name is not None else ''} "
+            f"Time elapse {time_elapsed}s ({self.end_time} - {self.start_time})")
+        return time_elapsed
 
 
 def time_function(func):
